
#include "accessdb.h"
#include "filedef.h"


PGconn *conn;

long  exec_sql_insert(char *);
static void export_fileinst(struct fileinstance *fi, long  procid, const char *);
static void export_variable(struct var *var, long  procid, const char *);
static void export_goal(struct file *f, long  procid);
static void export_cpids(struct pidlist*, long, const char *);
static void export_ord_deps(struct filelist *, long, struct makeprocess*, struct file *);
static void export_dbl_deps(struct fileinstlist *, long, struct makeprocess *, struct file *);
static void export_cpid(int pid, long  fioid, const char *);
static void export_dep(long, long, struct file*, struct file*,int,const char*, int);
static void export_makeps(struct makeprocess *ps);
static void update_makelevel(long procid, int level);

void die(int errorcode, char *msg)
{
	{ /* end transaction */
		PGresult *res;
		res = PQexec(conn, "ROLLBACK");
		PQclear(res);
	}
	
	switch (errorcode)
	{
		case 1: 
			printf("ERROR 001: Connection to database %s failed.\n", msg);
			break;
		case 2:
			printf("ERROR 002: %s. \n", msg);
			break;
		default:
			break;
	}
	printf("\t%s", PQerrorMessage(conn));
	printf("This operation failed, please retry\n");
	PQfinish(conn);
	abort();
}

void initdbconnection()
{
	/* backend connection */
	char *pghost = NULL, *pgport = NULL, *pgoptions = NULL, *pgtty = NULL;
	/*char *dbname = "GCC";*/
	/*char *dbname = "POSTGRESQL";*/
	/*char *dbname = "OTHERS";*/
	/*char *dbname = "MOZILLA";*/
	/*char *dbname = "HTTPD";*/
	/*char *dbname = "LINUX";*/
	char *dbname = "BTVDB";

	/* make a connection to the database */
	conn = PQsetdb(pghost, pgport, pgoptions, pgtty, dbname);
	
	/* check to see that the backend connection was successfully made */
	if (PQstatus(conn) == CONNECTION_BAD) die(1, dbname);
	printf("connected to dbname\n");
}

void enddbconnection()
{
	PQfinish(conn);
}

void export2db(struct makeprocess *ps)
{
	begintransaction();
	
	printf("saving make process %s:%d \n", ps->curdir, ps->pid);
	export_file_table();
	export_makeps(ps);

	endtransaction();
}

long  exec_sql_insert(char *sql)
{
	PGresult *res;
	long  oid;

	res = PQexec(conn, sql);
	checkinsertstatus (res);
	
	oid = PQoidValue(res);
	PQclear(res);

	/*
	if (oid == InvalidOid) 
	{
		die(2, "database not supporting oid");
	}*/
	return oid;
}

static void export_makeps(struct makeprocess *ps) 
{
	/* export the basic information of the process */
	char *sql = (char *) xmalloc(strlen (ps->buildname) + strlen(ps->curdir)
		       + strlen (ps->commandline) + 256);

	sprintf(sql, "INSERT INTO BTV_MAKEPROC VALUES\
			('%s', %d, %d, '%s', '%s')",
			ps->buildname, ps->pid, ps->ppid, 
			ps->curdir, ps->commandline);
	ps->procid = exec_sql_insert(sql);
	free (sql);

	{/* export files and their dependencies*/
		struct fileinstlist *ls;
		struct fileinstance *fi;
		
		for (ls = ps->files; ls; ls = ls->next)
			export_fileinst(ls->finst, ps->procid, ps->buildname);
		
		for (ls = ps->files; ls; ls = ls->next)
		{
			fi = ls->finst;
			export_cpids(fi->cpids, fi->finstid, ps->buildname);

			if (fi->num_dblcln == 0)
				export_ord_deps(fi->deps.ord, 
						fi->finstid, ps,fi->goal);
			else
				export_dbl_deps(fi->deps.dbl, 
						fi->finstid, ps,fi->goal);
		}
	}

	{/* export goals */
		struct filelist *ls;
		for (ls = ps->goals; ls; ls = ls->next) 
			export_goal(ls->f, ps->procid);
	}

	{/* export variables */
		struct varlist *ls;
		for (ls = ps->vars; ls; ls = ls->next) 
			export_variable(ls->v, ps->procid, ps->buildname);
	}
}

static char * get_parent_dir(const char * dir)
{
	char * res, * p;
	
	if (strcmp(dir, "/") == 0) return 0;

	
	res = (char *)xstrdup(dir);
	p = (char *)strrchr(res, '/');

	if (p) 
		*p = '\0';
	if (strlen(res) == 0) /* top root  directory */
	{
		*res = '/'; 
		*(res + 1) = '\0';
	}

	return res;
}

static void export_dir(const char * dir)
{
	char *sql = (char *)xmalloc(2 * strlen(dir) + 256);
	char *p_dir = (char *)xstrdup(dir);
 	char *c_dir, *p;
	PGresult *res;

	while (1)
	{
		c_dir = p_dir;
		p_dir = get_parent_dir(c_dir);

		if (!p_dir) 
		{
			free (c_dir);
			free (sql);
			return;
		}
		
		sprintf(sql, "SELECT * FROM BTV_DIR WHERE DIR_CHILD = '%s'", c_dir);
		res = PQexec(conn, sql);
		checkselectstatus(res);

		if (PQntuples(res) > 0 )
		{
			PQclear(res);
			free (c_dir);
			free (p_dir);
			return;
		}
		
		sprintf(sql, "INSERT INTO BTV_DIR VALUES('%s', '%s', FALSE)", p_dir, c_dir);
		exec_sql_insert(sql);
		free (c_dir);
	}
}

void export_file(struct file *f) 
{
	char *sql = (char *)xmalloc(strlen(f->name) + strlen(f->dir) + 256);
	PGresult *res;

	if (f->isexported) return;

	export_dir(f->dir);
	
	sprintf(sql, "SELECT OID \
			FROM BTV_FILE\
		       WHERE FILE_NAME = '%s' AND FILE_DIR = '%s'", 
			f->name, f->dir);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	if (PQntuples(res) > 0)
	{
		f->fileid = atol(PQgetvalue(res, 0, 0));
		f->isexported = 1;
		PQclear(res);
		return;
	}
	sprintf(sql, "INSERT INTO BTV_FILE VALUES('%s', '%s')", f->name, f->dir);
	f->fileid = exec_sql_insert(sql);
	f->isexported = 1;

	free (sql);
}

static char *str_prepare(char *str)
{
	int size = strlen(str);
	char *res = (char *)xmalloc(size + 512);
	char *r = res, *s = str;

	while (s < (str + strlen(str)) && *s) 
	{
		if (*s == '\n')
		{
			*r = ' ';
			r++; s++;
			continue;
		}
		
		if (*s == '\'') 
		{
			*r = '\'';
			r++;
		}
		else if (*s == '\\')
		{
			*r = '\\';
			r++;
		}
		*r = *s;
		r++; s++;
	}
	*r = '\0';
	return res;
}

static long find_dup_inst(struct fileinstance *fi, long  procid)
{
	PGresult * res;
	long oid;
	char sql[512];	
	char *cst_sql = "SELECT INST_CMDSTXT, INST_EXECMDS, OID\
			   FROM BTV_FILEINST \
			  WHERE INST_PROC = %d AND INST_FILE = %d \
			    AND INST_DBLCLN = %d "; 
	sprintf(sql, cst_sql, procid, fi->base->fileid, fi->double_colon);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	if (PQntuples(res) <= 0) return 0; 

	if (!PQgetisnull(res, 0, 0) && !PQgetisnull(res, 0, 1))
		return -1 * atol(PQgetvalue(res, 0, 2));
	
	oid = atol(PQgetvalue(res, 0, 2));
	PQclear(res);
	return oid;
}

static void export_fileinst(struct fileinstance *fi, long  procid, const char *bnm)
{
	char *sql;
	int size = strlen(fi->goal->name);
	long old_oid = find_dup_inst(fi, procid);

	if (old_oid > 0)
	{
		fi->finstid = old_oid;
		return;
	}
	else if (old_oid < 0)
	{
		PGresult *res;
		fi->finstid = -1 * old_oid;
		if (fi->cmdstxt && fi->execmds)
		{
			char *cmdstxt = str_prepare(fi->cmdstxt);
			char *execmds = str_prepare(fi->execmds);
			size += strlen(cmdstxt);
			size += strlen(execmds);
		
			sql = (char *)xmalloc(size + 512);
			sprintf(sql, "UPDATE BTV_FILEINST \
					 SET INST_CMDSTXT = '%s', INST_EXECMDS = '%s' \
				       WHERE OID = %d", cmdstxt, execmds, fi->finstid);
			free (cmdstxt);
			free (execmds);
		}
		else if (fi->cmdstxt)
		{
			char *cmdstxt = str_prepare(fi->cmdstxt);
			size += strlen(cmdstxt);
		
			sql = (char *)xmalloc(size + 512);
			sprintf(sql, "UPDATE BTV_FILEINST \
					 SET INST_CMDSTXT = '%s' \
				       WHERE OID = %d", cmdstxt, fi->finstid);
			free (cmdstxt);
		}
		else if (fi->execmds)
		{
			char *execmds = str_prepare(fi->execmds);
			size += strlen(execmds);
		
			sql = (char *)xmalloc(size + 512);
			sprintf(sql, "UPDATE BTV_FILEINST \
					 SET INST_EXECMDS = '%s' \
				       WHERE OID = %d", execmds, fi->finstid);
			free (execmds);
		}
		
		return;
	}
	
	/* old_oid  == 0 */
	if (fi->cmdstxt && fi->execmds)
	{
		char *cmdstxt = str_prepare(fi->cmdstxt);
		char *execmds = str_prepare(fi->execmds);
		size += strlen(cmdstxt);
		size += strlen(execmds);
		
		sql = (char *)xmalloc(size + 512);
		sprintf(sql, "INSERT INTO BTV_FILEINST VALUES (%d, %d, %d, %d,\
		        %d, %d, %d, '%s ','%s ','%s', '%s', '%s', '%s')",
			procid, fi->base->fileid, fi->double_colon, fi->num_dblcln,
			fi->istarget, fi->isphony, 0, cmdstxt, execmds, 
			fi->base->name, fi->base->dir, bnm, fi->goal->name);
		free (cmdstxt);
		free (execmds);

		
	}
	else if (fi->cmdstxt)
	{
		char *cmdstxt = str_prepare(fi->cmdstxt);
		size += strlen(cmdstxt);
		
		sql = (char *)xmalloc(size + 512);
		sprintf(sql,"INSERT INTO BTV_FILEINST VALUES (%d, %d, %d, %d,\
		       	%d, %d, %d, '%s ', NULL, '%s', '%s', '%s', '%s')",
			procid, fi->base->fileid, fi->double_colon, fi->num_dblcln,
			fi->istarget, fi->isphony, 0, cmdstxt, 
			fi->base->name, fi->base->dir, bnm, fi->goal->name);

		free (cmdstxt);
	}
	else if (fi->execmds)
	{
		char *execmds = str_prepare(fi->execmds);
		size += strlen(execmds);
		
		sql = (char *)xmalloc(size + 512);
		sprintf(sql, "INSERT INTO BTV_FILEINST VALUES (%d, %d, %d, %d,\
		       	%d, %d, %d, NULL, '%s ', '%s', '%s', '%s', '%s')",
			procid, fi->base->fileid, fi->double_colon, fi->num_dblcln,
			fi->istarget, fi->isphony, 0, execmds, 
			fi->base->name, fi->base->dir, bnm, fi->goal->name);
		free (execmds);
	}
	else
	{
		sql = (char *)xmalloc(512);
		sprintf(sql, "INSERT INTO BTV_FILEINST VALUES (%d, %d, %d, %d,\
		       	%d, %d, %d, NULL, NULL, '%s', '%s', '%s', '%s')",
			procid, fi->base->fileid, fi->double_colon, fi->num_dblcln,
			fi->istarget, fi->isphony, 0, 
			fi->base->name, fi->base->dir, bnm, fi->goal->name);
	}

	fi->finstid = exec_sql_insert(sql);
	
	free (sql);
}

static void export_variable(struct var *var, long  procid, const char *bnm)
{
	char *sql;
	char *value;
	int size = strlen(var->name) + strlen(var->value);
	
	sql = (char *)xmalloc(size + 512);
	
	value = str_prepare(var->value);
	sprintf(sql, "INSERT INTO BTV_VARIABLE VALUES(%d, '%s', '%s', %d, '%s')",
			procid, var->name, value, var->source, bnm);
	exec_sql_insert(sql);

	if (strncmp(var->name, "MAKELEVEL", 9) == 0)
		update_makelevel(procid, atoi(var->value));
	free (value);
	free (sql);
}

static void update_makelevel(long procid, int level)
{
	PGresult *res;
	char sql [512];
	
	sprintf(sql, "UPDATE BTV_MAKEPROC SET PS_MAKELEVEL = %d WHERE OID = %d", 
			level, procid);
	res = PQexec(conn, sql);
	checkupdatestatus(res);
	PQclear(res);
}

static void export_goal(struct file *f, long  procid)
{
	PGresult *res;
	char sql[512];

	sprintf(sql, "UPDATE BTV_FILEINST\
			 SET INST_ISGOAL = 1\
		       WHERE INST_PROC = %d \
		         AND INST_FILE = %d \
		         AND INST_DBLCLN = 0 \
			 AND INST_GOAL = INST_NAME", 
			procid, f->fileid);
	res = PQexec(conn, sql);
	checkupdatestatus(res);

	if (atoi(PQcmdTuples(res)) != 1)
	{
		PQclear(res);
		die(2, "the goal doesn't exist");
	}
	PQclear(res);
}

static void export_cpids(struct pidlist *cpids, long fioid, const char *bnm)
{
	struct pidlist *ls;
	char sql[512];
	
	for (ls = cpids; ls; ls = ls->next)
	{
		sprintf(sql, "INSERT INTO BTV_CPID VALUES (%d, %d, '%s')", 
			fioid, ls->pid, bnm);
		exec_sql_insert(sql);
	}
}

static void export_ord_deps(struct filelist *ls, long fioid, 
		struct makeprocess *ps, struct file *g)
{
	struct filelist *fls;
	for (fls = ls; fls; fls = fls->next)
		export_dep(fioid, ps->procid, fls->f, g, 0, 
				ps->buildname, fls->order);
}

static void export_dbl_deps(struct fileinstlist *ls, long fioid, 
		struct makeprocess *ps, struct file *g)
{
	struct fileinstlist *fls;
	for (fls = ls; fls; fls = fls->next)
		export_dep(fioid, ps->procid, fls->finst->base, g,
				fls->finst->double_colon, 
				ps->buildname, fls->order);
}

static void export_dep(long fioid, long procid, struct file *f, struct file *g,
		int dblcln, const char *bnm, int order)
{
	PGresult *res;
	char sql[512];
	long  finstid;

	sprintf(sql, "SELECT OID\
			FROM BTV_FILEINST\
		       WHERE INST_PROC = %d\
		         AND INST_FILE = %d\
			 AND INST_GOAL = '%s'\
			 AND INST_DBLCLN = %d" ,
			procid, f->fileid, g->name, dblcln);

	res = PQexec(conn, sql);
	checkselectstatus(res);

	if (PQntuples(res) <= 0)
	{
		PQclear(res);
		sprintf(sql, "INSERT INTO BTV_FILEINST VALUES (%d, %d, 0, 0,\
		        0, 0, 0, NULL, NULL, '%s', '%s', '%s', '%s')",
			procid, f->fileid, f->name, f->dir, bnm, g->name);
		finstid = exec_sql_insert(sql);
	}
	else if (PQntuples(res) == 1) {
		finstid = atol(PQgetvalue(res, 0, 0));
		PQclear(res);
	}
	else
		die(2, "FILEINST PRIMARY KEY CONFLICT");
	
	sprintf(sql, "INSERT INTO BTV_DEPENDENCY VALUES (%d, %d, %d, '%s')", 
			fioid, finstid, order, bnm);
	exec_sql_insert(sql);
}

char *removelog(const char *bnm)
{
	PGresult *res;
	char sql[512];
	int num;
	
	printf("REMOVING LOG %s ... \n", bnm);

	sprintf(sql, "DELETE FROM BTV_CALL WHERE CALL_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);
	
	sprintf(sql, "DELETE FROM BTV_CPID WHERE CPID_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);

	sprintf(sql, "DELETE FROM BTV_DEPENDENCY WHERE DEP_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);
	
	sprintf(sql, "DELETE FROM BTV_FILEINST WHERE INST_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);
	
	sprintf(sql, "DELETE FROM BTV_MAKEPROC WHERE PS_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);
	
	sprintf(sql, "DELETE FROM BTV_VARIABLE WHERE VAR_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	num = atoi(PQcmdTuples(res));
	PQclear(res);
	
	printf("LOG %s HAS BEEN REMOVED ! \n", bnm);
}

int isloaded(const char *bnm)
{
	PGresult *res;
	char sql[256];
	char ans[10];

	begintransaction();
	sprintf(sql, "SELECT * FROM BTV_MAKEPROC WHERE PS_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	if (PQntuples(res) <= 0) {
		PQclear(res);
		endtransaction();
		return 0;
	}
	PQclear(res);
	endtransaction();
		
	printf("\nLog %s has ALREADY been loaded !!!\n", bnm);
	printf("REMOVE it? (yes/no)");
	scanf("%s", ans);

	if (strncmp(ans, "yes", 3) != 0) return 1;

	begintransaction();
	removelog(bnm);
	endtransaction();
	return 0;
}

