
#include <assert.h>
#include <stdlib.h>
#include "makeps.h"
#include "util.h"

struct makeprocess *_makeprocess(const char *bnm)
{
	struct makeprocess *newps;
	newps = (struct makeprocess*)xmalloc(sizeof(*newps));
	newps->buildname = (char *)xstrdup(bnm);
	newps->procid = 0;
	newps->commandline = 0;
	newps->curdir = 0;
	newps->vars = 0;
	newps->files = 0;
	newps->goals = 0;
	return newps;
}

void ps_free(struct makeprocess *ps)
{
	free (ps->buildname);
	free (ps->commandline);
	free (ps->curdir);
	varlist_free (ps->vars);
	flist_free (ps->files);
	filelist_free (ps->goals);
	free (ps);
}

struct makepslist *_makepslist(struct makepslist *head, struct makeprocess *p)
{
	struct makepslist *result = (struct makepslist *)xmalloc(sizeof(*result));
	result->ps = p;
	result->next = head;
	return result;
}

void pslist_free(struct makepslist *ls)
{
	struct makepslist *head;

	while (ls)
	{
		head = ls; 
		ps_free(head->ps);
		ls = ls->next;
		free(head);
	}
}

void ps_writeout(struct makeprocess *ps)
{
	if (!ps) return;
	
	printf("make process %d\n", ps->procid);
	printf("commandline = %s\n", ps->commandline);
	printf("curdir = %s\n", ps->curdir);
	printf("pid = %d, ppid = %d \n", ps->pid, ps->ppid);
	vl_writeout(ps->vars);
	flist_writeout(ps->files);
	filelist_writeout(ps->goals);
}

void ps_setcommandline(struct makeprocess *ps, char *content, int len)
{
	char *start = content;
	assert(ps);
	
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	
	ps->commandline = xstrncpy(start, len - (start - content));
}

void ps_setcurdir(struct makeprocess *ps, char *content, int len)
{
	char *start = content;
	assert(ps);
	
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	
	if (content[len-1] == '/') 
		ps->curdir = xstrncpy(start, len - (start - content) - 1);
	else
		ps->curdir = xstrncpy(start, len - (start - content));
}

void ps_setpid(struct makeprocess *ps, char *content, int len)
{
	char *start = content;
	assert(ps);
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	ps->pid = atoi(start);
}

void ps_setppid(struct makeprocess *ps, char *content, int len)
{
	char *start = content;
	assert(ps);
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	ps->ppid = atoi(start);
}

enum varsource ps_addvariable(struct makeprocess *ps, char *content, int len, enum varsource vs)
{
	char *start = content;
	char *end, *name, *value;

	assert(ps);
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	
	if (*start == '#') 
	{
		switch (*(start + 1))
		{
			case 'e': return o_env;
			case 'd': return o_default;
			case 'm': return o_file;
			case 'c': return o_command;
			case 'o': return o_override;
			case 'a': return o_automatic;
			default: return o_invalid;
		}
	}
	/* find name */
	end = start;
	while (*end != '=' && end < (content + len)) end++;
	
	if (*end != '=') 
	{
		printf("FATAL: no equal find in the variable definition. \n");
		assert (0);
	}

	name = xstrncpy(start, end - start);

	start = end + 1;
	end = start;
	while (end < (content + len)) end++;
	
	value = xstrncpy(start, end - start);

	ps->vars = _varlist(ps->vars, name, value, vs);
	return o_invalid;
}

struct fileinstance * ps_addfileinst(struct makeprocess *ps, struct file *goal, char *content, int len)
{
	char *start = content;
	char *name;
	struct fileinstance *finst;

	assert(ps);
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	name = xstrncpy(start, len - (start - content));

	finst = _fileinstance(locatefile(name, ps->curdir), goal);
	ps->files = _fileinstlist(ps->files, finst);
	free(name);
	return finst;
}

struct file *ps_addgoal(struct makeprocess *ps, char *content, int len)
{
	char *start = content;
	char *name;
	struct file *f;

	assert (ps);
	while (*start == '\t' || *start == ' ' || *start == '\n') start ++;
	name = xstrncpy(start, len - (start - content));

	f = locatefile(name, ps->curdir);
	ps->goals = _filelist(ps->goals, f); 
	free (name);
	return f;
}
