
#include "accessdb.h"
#include "util.h"

#define GOOD_MATCH	0.6
#define PID_MATCH	5

typedef void (*func_prepare)  (PGresult *res, const char *bnm);

static void prepare(func_prepare funcp, const char *bnm, int cont);
static void setinvoke_oneps(long, int, int, const char *, const char *, 
		int, const char *, const char *, int);
static void setcaller(const char *version, long caller, long proc);
static double cal_matchrate(const char *, const char *, const char *, 
		const char *, const char *);
static int sort_matchrate(int num, int *matches, double *matchrates);
static int isprepared(const char *bnm);
static void setinvoke_allps(PGresult *res, const char *bnm);
static void reviewinvoke_allps(PGresult *res, const char *bnm);
static char *show_object(int oid, int pid, int ppid, const char *dir, 
		const char *cmdline, int makelevel);
static long show_matched_object(long oid, const char *bnm);
static void delete_invoke(long call_oid);

static int perfect_match;
static int only_match;
static int user_match;
static int user_undecided;
static int changed_match;

static void init_match_stat()
{
	perfect_match = 0;
	only_match = 0;
	user_match = 0;
	user_undecided = 0;
}

static void show_match_stat()
{
	int total_match = perfect_match + only_match
	       	+ user_match + user_undecided;
	printf("\n Set Invoke relation statistics:\n");
	printf("\n \tTotal Match = %d, in which there are\n", total_match);
	printf("\n \t\t%d Perfect Match (%f%%)", 
			perfect_match, (double)perfect_match*100/total_match);
	printf("\n \t\t%d Only Match (%f%%)", 
			only_match, (double)only_match*100/total_match);
	printf("\n \t\t%d User Match (%f%%)", 
			user_match, (double)user_match*100/total_match);
	printf("\n \t\t%d Undeterminated (%f%%)", 
			user_undecided, (double)user_undecided*100/total_match);
	printf("\n");
}

static char * get_buildversion(const char *prog)
{
	char *res = (char *)xmalloc(256);
	printf("%s: Please input system building name [e.g. pgsql] =>", prog);
	scanf("%s", res);
	return res;
}

void setinvoke()
{
	char *bnm = get_input_string
		("SETINVOKE: Please input system building name [e.g. pgsql]");

	begintransaction();

	if (isprepared(bnm)) 
	{
		printf("Do nothing\n");
		endtransaction();
		return;
	}

	printf("\nSetting invoke relation for %s ... \n", bnm);
	prepare(setinvoke_allps, bnm, 0);
	printf("Setting invoke relation is completed.\n");

	endtransaction();
	free (bnm);
}

void reviewinvoke()
{
	char *bnm = get_input_string
		("REVIEWINVOKE: Please input system building name [e.g. pgsql]");

	begintransaction();

	printf("\nreviewing invoke relation for %s ... \n", bnm);
	prepare(reviewinvoke_allps, bnm, 0);
	printf("reviewing invoke relation is completed.\n");

	endtransaction();
	free (bnm);
}

void continvoke()
{
	char *bnm = get_input_string
		("CONTINVOKE: Please input system building name [e.g. pgsql]");

	begintransaction();

	printf("\nContinue setting invoke relation for %s ... \n", bnm);
	prepare(reviewinvoke_allps, bnm, 1);
	printf("Setting invoke relation is completed.\n");

	endtransaction();
	free (bnm);
}

static int isprepared(const char *bnm)
{
	PGresult *res;
	char sql[256];
	char ans[10];

	sprintf(sql, "SELECT * FROM BTV_CALL WHERE CALL_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	if (PQntuples(res) <= 0) {
		PQclear(res);
		return 0;
	}
	PQclear(res);
	
	printf("The invoke relation for %s have ALREADY been set. \n", bnm);
	printf("RESET it now? (yes/no) ");
	scanf("%s", ans);

	if (strncmp(ans, "yes", 3) != 0) return 1;

	sprintf(sql, "DELETE FROM BTV_CALL WHERE CALL_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	
	PQclear(res);
	return 0;
}

static void prepare(func_prepare funcp, const char *bnm, int cont)
{
        PGresult *res;
	char sql[512];
	int ntup;
	
	if (!cont)
		sprintf(sql, "SELECT OID, PS_PID, PS_PPID, PS_DIR, \
				     PS_CMDLINE, PS_MAKELEVEL\
				FROM BTV_MAKEPROC \
			       WHERE PS_VERSION = '%s'\
			       ORDER BY PS_PID", bnm);
	else
		sprintf(sql, "SELECT OID, PS_PID, PS_PPID, PS_DIR, \
				     PS_CMDLINE, PS_MAKELEVEL\
				FROM BTV_MAKEPROC\
			       WHERE PS_VERSION = '%s'
			         AND OID NOT IN (\
				      SELECT INST_PROC\
				        FROM BTV_CALL, BTV_FILEINST\
				       WHERE CALL_CALLEE = BTV_FILEINST.OID\
				         AND CALL_VERSION = '%s')", bnm, bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	ntup = PQntuples(res);
	if (ntup < 1)
	{
		printf("There is not make process of %s\n", bnm);
		printf("Nothing is done.\n");
	}
	else if (ntup == 1)
	{
		printf("\nOnly ONE make process of %s is found.\n",bnm);
		printf("Dependency relation is ALREADY COMPLETE!\n");
		printf("Nothing is done.\n");
	}
	else 
	{
		funcp(res, bnm);
        }
	PQclear(res);
}

static void setinvoke_allps(PGresult *res, const char *bnm)
{
	int i;
	long oid, pid, ppid;
	char *dir, *cmdline, *goal;
       	int makelevel;
	
	init_match_stat();
	for (i = 0; i < PQntuples(res); i++)
	{
		oid = atol(PQgetvalue(res, i, 0));
		pid = atoi(PQgetvalue(res, i, 1));
		ppid = atoi(PQgetvalue(res, i, 2));
		dir = PQgetvalue(res, i, 3);
		cmdline = PQgetvalue(res, i, 4);
		makelevel = atoi(PQgetvalue(res, i, 5));
		goal = show_object(oid, pid, ppid, dir, cmdline, makelevel);
		setinvoke_oneps(oid, pid, ppid, dir, cmdline, 
				makelevel, bnm, goal, 0);
		free (goal);
	}
	show_match_stat();
}

static void reviewinvoke_allps(PGresult *res, const char *bnm)
{
	int i;
	long oid, pid, ppid, call_oid;
	char *dir, *cmdline, *goal;
       	int makelevel;
	char choice[256];
	int change_num = 0;
	
	for (i = 0; i < PQntuples(res); i++)
	{
		oid = atol(PQgetvalue(res, i, 0));
		pid = atoi(PQgetvalue(res, i, 1));
		ppid = atoi(PQgetvalue(res, i, 2));
		dir = PQgetvalue(res, i, 3);
		cmdline = PQgetvalue(res, i, 4);
		makelevel = atoi(PQgetvalue(res, i, 5));

		if (makelevel == 0) continue;
		goal = show_object(oid, pid, ppid, dir, cmdline, makelevel);
		call_oid = show_matched_object(oid, bnm);
		
		scanf("%s", choice);
		if (strncmp(choice, "quit", 4) == 0) break;
		if (strncmp(choice, "yes", 3) != 0) continue;
		
		delete_invoke(call_oid);
		setinvoke_oneps(oid, pid, ppid, dir, cmdline, 
				makelevel, bnm, goal, 1);
		change_num ++;
	}
	printf("Total %d INVOKE relation are changed.\n", change_num);
}

static void delete_invoke(long call_oid)
{
	PGresult *res;
	char sql[512];
	
	if (call_oid < 0) return;
	
	sprintf(sql, "DELETE FROM BTV_CALL WHERE OID = %d", call_oid);
	res = PQexec(conn, sql);
	checkdeletestatus(res);
	PQclear(res);
}

static long show_matched_object(long oid, const char *bnm)
{
	PGresult *res;
	char sql[512];
	long res_oid;

	sprintf(sql, "SELECT CALLER.INST_DIR, CALLER.INST_NAME, \
			     CALLER.INST_CMDSTXT, CALLER.INST_EXECMDS, \
			     BTV_CALL.OID \
			FROM BTV_CALL, \
			     BTV_FILEINST AS CALLER, \
			     BTV_FILEINST AS CALLEE \
		       WHERE CALL_VERSION = '%s'\
		         AND CALL_CALLER = CALLER.OID \
			 AND CALL_CALLEE = CALLEE.OID \
		         AND CALLEE.INST_PROC = %d", bnm, oid);
	res = PQexec(conn, sql);
	checkselectstatus(res);
	
	if (PQntuples(res) <= 0)
	{
		printf("\nThe INVOKE relation has NOT been set. \n");
		printf("Do you want to set it now?(yes/no)");
		res_oid = -1;
	}
	else if (PQntuples(res) > 1)
	{
		printf("more that one goal is found\n");
		abort();
	}
	else
	{
		printf("\nCURRENT CHOICE: %s/%s\n", 
				PQgetvalue(res, 0, 0), PQgetvalue(res, 0, 1));
		printf("---------------------------------------------------\n");
		printf("\nCOMMAND LINE:\n%s\n", PQgetvalue(res, 0, 2));
		printf("---------------------------------------------------\n");
		printf("\nEXECUTED CMD:\n%s\n", PQgetvalue(res, 0, 3));
		printf("===================================================\n");
		printf("\nDo you want to change it? (yes/no)");
		res_oid = atol(PQgetvalue(res, 0, 4));
	}
	PQclear(res);
	return res_oid;
}

static int pid_matched(long oid, int pid, int ppid)
{
	PGresult *res;
	char sql[512];
	int ntup, cpid;
	
	sprintf(sql, "SELECT CPID_CPID FROM BTV_CPID\
		       WHERE CPID_FILEINST = %d", oid);
	res = PQexec(conn, sql);
	checkselectstatus(res);
	
	for (ntup = 0; ntup < PQntuples(res); ntup++)
	{
		cpid = atoi(PQgetvalue(res, ntup, 0));
		if (cpid == pid || cpid == ppid)
		{
			PQclear(res);
			return 1;
		}
	}
	PQclear(res);
	return 0;
}

static char *show_object(int oid, int pid, int ppid, const char *dir, 
		const char *cmdline, int makelevel)
{
	PGresult *res;
	char sql[1024];
	int ntup;
	char *goal;
	
	sprintf(sql, "SELECT INST_NAME FROM BTV_FILEINST \
		       WHERE INST_PROC = %d AND INST_ISGOAL != 0", oid);
	res = PQexec(conn, sql);
	checkselectstatus(res);
	goal = (char *)xstrdup(PQgetvalue(res, 0, 0));

	printf("\n");
	printf("***********************************************************\n");
	printf("*  Find caller for makeproc with %d goals\n", PQntuples(res));
	printf("*\tpid = %d; ppid = %d\n", pid, ppid);
	printf("*\tdir = %s\n", dir);
	printf("*\tcommand line = %s\n", cmdline);
	for (ntup = 0; ntup < PQntuples(res); ntup++)
		printf("*\tgoal[%d] = %s\n", ntup+1, PQgetvalue(res, ntup, 0));
	printf("*\tMAKELEVEL = %d\n", makelevel);
	printf("***********************************************************\n");
	PQclear(res);
	return goal;
}

static show_match_file(int ord, double matchrate, char *dir,
		char *filenm, int pid, char *cmdstxt, char *execmds)
{
	int j;
	
	if (matchrate < PID_MATCH)
	{
		printf("\n(%f%%)\n", matchrate*100);
	}
	else
	{
		matchrate -= PID_MATCH;
		printf("\n(%f%%)  MATCHED BY PROCESS ID !!!\n", matchrate*100);
	}
	for (j = 1; j < (matchrate + 0.05) * 5; j ++) printf("*");
	for (; j <= 5; j++) printf("-");

	printf("[%d] %s/%s  PID = %d\n", 
			ord, dir, filenm, pid);
	
	printf("\nCOMMAND LINE:\n%s\n", cmdstxt);
	printf("\nEXECUTED CMD:\n%s\n", execmds);
	printf("-----------------------------------------------------\n");
}

static void setinvoke_oneps(long oid, int pid, int ppid, const char *dir, 
		const char *cmdline, int makelevel, const char *bnm, 
		const char *goal, int need_confirm)
{
	PGresult *res;
	char sql[1024], choice[256];
	int i, ntup, choicenum, first_match, matchnum;
	int *matches; double *matchrates;
	const int step = 1;
	
	/* Step 0: print make process information */

	if (makelevel == 0)	/* top process */
	{
		printf("No suggested caller process.  \n");
		printf("This process is the top make process. \n");
		return;
	}

	/* Step 1: get possible caller from its upper process (makelevel-1) */
	sprintf(sql, "SELECT BTV_FILEINST.OID, INST_NAME, INST_CMDSTXT,\
			     INST_EXECMDS, INST_PROC, PS_PID, INST_DIR\
			FROM BTV_FILEINST, BTV_MAKEPROC\
		       WHERE INST_PROC = BTV_MAKEPROC.OID\
		         AND INST_ISTARGET != 0\
			 AND INST_EXECMDS IS NOT NULL\
			 AND PS_VERSION = '%s'\
			 AND PS_MAKELEVEL = %d ", bnm, makelevel-1);
	
	res = PQexec(conn, sql);
	
	if (PQntuples(res) <= 0)
	{
		PQclear(res);
		printf("WARNING: No suggested caller process.  \n");
		printf("WARNING: There maybe something wrong. \n");
		printf("Press any key to continue");
		scanf("%s", choice);
		return;
	}

	/* Step 2: calculate match rate(between command line and execmds) 
	 *         for each candidates */
	ntup = PQntuples(res);
	matches = (int *)xmalloc(ntup * sizeof(int));
	matchrates = (double *)xmalloc(ntup * sizeof(double));
	
	for (i = 0; i < ntup ; i++)
	{
		matches[i] = i;
		matchrates[i] = cal_matchrate(dir, goal, cmdline, 
				PQgetvalue(res, i, 1),
				PQgetvalue(res, i, 3));
		if (pid_matched(atoi(PQgetvalue(res, i, 0)), pid, ppid))
			matchrates[i] += PID_MATCH;
	}

	matchnum = sort_matchrate(ntup, matches, matchrates);

	/* Step 3: Select the best match
	 *         -- high match rate and can be traced by process id */
	for (i = 0; i < matchnum; i++)
	{
		if (matchrates[i] < GOOD_MATCH + PID_MATCH) continue;
		
		printf("\n Find Perfect Match !!!\n");

		if (!need_confirm)
			setcaller(bnm,atol(PQgetvalue(res, matches[i], 0)),oid);
		else
		{
			show_match_file(1, matchrates[0], 
					PQgetvalue(res, 0, 6), 
					PQgetvalue(res, 0, 1), 
					atoi(PQgetvalue(res, 0, 5)),
					PQgetvalue(res, 0, 2),
					PQgetvalue(res, 0, 3));
			printf("accept the perfect match ?(yes/no)");
			scanf("%s", choice);
			if (strncmp(choice, "yes", 3) != 0) continue;
			setcaller(bnm,atol(PQgetvalue(res, matches[i], 0)),oid);
		}
		free (matches);
		free (matchrates);
		PQclear(res);
		perfect_match++;
		return;
	}

	/* Step 5: Only match */
	if (ntup == 1)
	{
		show_match_file(1, matchrates[0], 
				PQgetvalue(res, 0, 6), 
				PQgetvalue(res, 0, 1), 
				atoi(PQgetvalue(res, 0, 5)),
				PQgetvalue(res, 0, 2),
				PQgetvalue(res, 0, 3));
		if (!need_confirm)
		{
			printf("\n Find Only Match !!!");
			setcaller(bnm, atol(PQgetvalue(res, 0, 0)), oid);
			only_match++;
			free (matches);
			free (matchrates);
			PQclear(res);
			return;
		}
		printf("\n Find Only Match !!! accept it:(ues/no)");
		scanf("%s", choice);
		if (strncmp(choice, "yes", 3) != 0)
			user_undecided++;
		else
		{
			setcaller(bnm, atol(PQgetvalue(res, 0, 0)), oid);
			only_match++;
		}

		free (matches);
		free (matchrates);
		PQclear(res);
		return;
	}

	/* Step 6: If can not solve the match automatically
	 * 	   List all possible candidates and Let user to 
	 * 	   make the desicion */
	printf("\nPossible candidates are :\n");
	
	first_match = 0;
	while(1)
	{
		for (i = first_match; i < ntup && i < (first_match + step); i++)
		{
			show_match_file(i+1, matchrates[i], 
					PQgetvalue(res, matches[i], 6), 
					PQgetvalue(res, matches[i], 1), 
					atoi(PQgetvalue(res, matches[i], 5)),
					PQgetvalue(res, matches[i], 2),
					PQgetvalue(res, matches[i], 3));
		}

		printf("\nPress 'p' to see previous choice; ");
		printf("\nPress 'n' to see more choice; ");
		printf("\nPress 'q' to skip;");
		printf("\nPress number to select [1 - %d,n,p,q]: ", i);
		scanf("%s", choice);
		
		if (choice[0] == 'q') 
		{
			user_undecided ++;
			break;
		}

		if (choice[0] == 'n') 
		{
			if (first_match + step>= ntup)
				first_match = ntup - step;
			else
				first_match += step;
			continue;
		}

		if (choice[0] == 'p')
		{
			if (first_match - step < 0)
				first_match = 0;
			else
				first_match -=step;
			continue;
		}
		
		choicenum = atol(choice);
		/*
		if (!(choicenum >= 1 && choicenum <= i))
			
			printf("\nBAD choice\n");
		else*/
		{
			setcaller(bnm, atol(PQgetvalue(res, matches[choicenum-1], 0)), oid);
			user_match++;
			break;
		}
	}
	
	free (matches);
	free (matchrates);
	PQclear(res);
}

static int find_match(const char *str, const char *word, const char *fmt)
{
	const char *str_end  = str + strlen(str);
	const char *word_end = word + strlen(word);
	const char *str_cur  = str;
	const char *word_cur;
	char format[10];

	sprintf(format, fmt);

	while (str_cur < str_end)
	{
		/* get first char not in format */
		while (str_cur < str_end)
		{
			if (!strchr(format, *str_cur)) break;
			str_cur++;
		}

		/* compare word and str */
		word_cur = word;
		while (word_cur < word_end && str_cur < str_end 
				&& *str_cur == *word_cur)
		{
			word_cur++; str_cur++;
		}
		if (word_cur == word_end && strchr(format, *str_cur))
			return 1;

		/* get first char in format */
		while (str_cur < str_end )
		{
			if (strchr(format, *str_cur)) break;  
			str_cur ++;
		}
		str_cur++; 
	}
	return 0;
}

static double cal_matchrate(const char *dir, const char *goal, 
		const char *cmdline, const char *target, const char *execmds)
{
	char *tmp_cmdline = (char *)xstrdup(cmdline);
	char *token;
	int total = 0, match = 0;

	token = (char *)strrchr(dir, '/');
	if (token && strlen(token) > 1)
	{
		if (find_match(execmds, (token +1), " /\t\n")) 
		{
			match++; total++;
		}
		if (find_match(target, (token + 1), " -")) 
		{
			match++; total++;
		}
	}

	if (find_match(execmds, goal, " -\t\n")) {match ++; total++;}
	
	token = (char *)strtok(tmp_cmdline, " =\t\n");
	while (token)
	{
		if (find_match(execmds, token, " =\"\'\t\n")) match++;
		total++;
		token = (char *)strtok(NULL, " =\t\n");
	}

	free (tmp_cmdline);
	return (double)match/total;
}

static void setcaller(const char *version, long caller, long proc)
{
        char sql[512];
        sprintf(sql, "INSERT INTO BTV_CALL\
				( SELECT '%s', %d, BTV_FILEINST.OID\
				    FROM BTV_FILEINST\
				   WHERE INST_PROC = %d\
				     AND INST_ISGOAL != 0)", 
			version, caller, proc);
        exec_sql_insert(sql);
	printf("\nInsert invoke relation successfully !!!\n\n");
}

static int sort_matchrate(int num, int *matches, double *matchrates)
{
	int i, j, tmpint;
	double tmp;
	
	for (i = 0; i < num - 1; i++)
	{
		for (j = i+1; j < num; j++)
		{
			if (matchrates[i] < matchrates[j])
			{
				tmp = matchrates[i];
				matchrates[i] = matchrates[j];
				matchrates[j] = tmp;

				tmpint = matches[i];
				matches[i] = matches[j];
				matches[j] = tmpint;
			}
		}
	}
	
	i = 0; j = 0;
	
	while (i < num && matchrates[i] > GOOD_MATCH)
	{
		if (matchrates[i] < PID_MATCH) 
			j++;
		else if (matchrates[i] > PID_MATCH + GOOD_MATCH) 
			j++;
		i++;
	}

	return j;
}

#undef PID_MATCH
#undef GOOD_MATCH
