
#include <stdio.h>
#include "accessdb.h"
#include "util.h"
#include "torsf.h"

static db2rsf(const char *filenm, const char *bnm);

void toRSF()
{
	char *filenm = get_input_string
		("DBtoRSF: Please input RSF filename [e.g. output.rsf]");
	char *bnm = get_input_string
		("DBtoRSF: Please input system building name [e.g. pgsql]");

	printf("\nExporting %s from database to %s ...\n", bnm, filenm);
	db2rsf(filenm, bnm);
	printf("End of export %s to %s \n\n", bnm, filenm);
	
	free (filenm);
	free (bnm);
}

static int exist_version(const char *bnm)
{
	char sql[512];
	PGresult *res;
	
	sprintf(sql, "SELECT * FROM BTV_MAKEPROC WHERE PS_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);
	
	if (PQntuples(res) <= 0) 
	{
		printf("no system building %s \n", bnm);
		printf("please load facts first \n");
		PQclear(res);
		return 0;
	}
	PQclear(res);
	return 1;
}

static db2rsf(const char *filenm, const char *bnm)
{
	FILE *fp;

	fp = fopen(filenm, "w");
	if (!fp)
	{
		printf("Error: can not open %s !!!\n ", filenm);
		abort();
	}

	begintransaction();
	
	/* not safe for multiple user */
	if (exist_version(bnm)) 
	{
		torsf_dir(bnm, fp);
		torsf_file(bnm, fp);
		torsf_dependency(bnm, fp);
		torsf_call(bnm, fp);
	}

	endtransaction();
	fclose(fp);
}

static void choose_dirs(const char *bnm)
{
	char sql[512];
	PGresult *res;
	char *cst_sql = "UPDATE BTV_DIR \
			    SET DIR_CHOOSED = TRUE \
			  WHERE DIR_CHILD IN (\
			       SELECT DISTINCT INST_DIR \
			         FROM BTV_FILEINST\
			        WHERE INST_VERSION = '%s')";

	sprintf(sql, "UPDATE BTV_DIR SET DIR_CHOOSED=FALSE WHERE DIR_CHOOSED");
	res = PQexec(conn, sql);
	checkupdatestatus(res);
	PQclear(res);

	sprintf(sql, cst_sql, bnm);

	res = PQexec(conn, sql);
	checkupdatestatus(res);

	while (atoi(PQcmdTuples(res)) > 0)
	{
		PQclear(res);
		
		sprintf(sql, "UPDATE BTV_DIR\
				 SET DIR_CHOOSED = TRUE \
			       WHERE DIR_CHOOSED = FALSE \
			         AND DIR_CHILD IN (\
					 SELECT DIR_PARENT\
					   FROM BTV_DIR\
					  WHERE DIR_CHOOSED)");
		res = PQexec(conn, sql);
		checkupdatestatus(res);
	}
	PQclear(res);
}

static void torsf_root(FILE *fp)
{
	PGresult *res;
	int ntup;
	char sql [512] = "SELECT DISTINCT DIR_PARENT\
			    FROM BTV_DIR \
			   WHERE DIR_CHOOSED AND DIR_PARENT NOT IN \
			     (SELECT DIR_CHILD FROM BTV_DIR WHERE DIR_CHOOSED)";
	res = PQexec(conn, sql);
	checkselectstatus(res);
	ntup = PQntuples(res);

	if (ntup == 1)
		fprintf(fp, "ROOT\t%s\t0\n", PQgetvalue(res, 0, 0));
	else 
	{
		PQclear(res);
		die(2, "more than one ROOT is found");
	}
	PQclear(res);
}

void torsf_dir(const char *bnm, FILE *fp)
{
	char sql[512];
	PGresult *res;
	int ntup;
	
	choose_dirs(bnm);
	torsf_root(fp);

	sprintf(sql, "SELECT DIR_PARENT, DIR_CHILD\
		       	FROM BTV_DIR WHERE DIR_CHOOSED");
	res = PQexec(conn, sql);
	checkselectstatus(res);

	for (ntup = 0; ntup < PQntuples(res); ntup ++)
	{
		fprintf(fp, "CONTAIN\t%s\t%s\n", 
			PQgetvalue(res, ntup, 0), PQgetvalue(res, ntup, 1));
	}

	PQclear(res);
}

void torsf_file(const char *bnm, FILE *fp)
{
	char sql[512];
	PGresult *res;
	int ntup;

	sprintf(sql, "SELECT BTV_FILEINST.OID, INST_NAME, INST_DIR, \
			     INST_ISPHONY, INST_ISTARGET, INST_ISGOAL,\
			     INST_EXECMDS, INST_ISDBLCLN, INST_DBLCLN, \
			     INST_CMDSTXT \
			FROM BTV_FILEINST, BTV_MAKEPROC \
		       WHERE INST_PROC = BTV_MAKEPROC.OID \
		         AND PS_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	for (ntup = 0; ntup < PQntuples(res); ntup++)
	{
		int fid = atol(PQgetvalue(res, ntup, 0));
		
		fprintf(fp, "FILE\t%d\t%s\n", fid, PQgetvalue(res, ntup, 1));
		fprintf(fp, "CONTAIN\t%s\t%d\n", PQgetvalue(res, ntup, 2), fid);
		
		if (atoi(PQgetvalue(res, ntup, 3)) != 0)
			fprintf(fp, "PHONY\t%d\t0\n", fid);
		/*if (atoi(PQgetvalue(res, ntup, 4)) != 0)*/
		if (!PQgetisnull(res, ntup, 9))
			fprintf(fp, "TARGET\t%d\t0\n", fid);
		if (atoi(PQgetvalue(res, ntup, 5)) != 0)
			fprintf(fp, "GOAL\t%d\t0\n", fid);
		if (!PQgetisnull(res, ntup, 6))
			fprintf(fp, "EXECUTED\t%d\t0\n", fid);
		if (atoi(PQgetvalue(res, ntup, 7)) != 0)
			fprintf(fp, "DBLCLN\t%d\t0\n", fid);
		else if (atoi(PQgetvalue(res, ntup, 8)) != 0)
			fprintf(fp, "DBLCLN\t%d\t%d\n", fid,
					atoi(PQgetvalue(res, ntup, 8)));
	}
	
	PQclear(res);
}

void torsf_dependency(const char *bnm, FILE *fp)
{
	char sql[512];
	PGresult *res;
	int ntup;

	sprintf(sql, "SELECT DEP_TARGET, DEP_SOURCE \
			FROM BTV_DEPENDENCY \
		       WHERE DEP_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	for (ntup = 0; ntup < PQntuples(res); ntup++)
	{
		fprintf(fp, "DEPENDON\t%d\t%d\n", atol(PQgetvalue(res, ntup, 0)), 
				atol(PQgetvalue(res, ntup, 1)));
	}
	PQclear(res);
}

void torsf_call(const char *bnm, FILE *fp)
{
	char sql[512];
	PGresult *res;
	int ntup;

	sprintf(sql, "SELECT CALL_CALLER, CALL_CALLEE \
			FROM BTV_CALL \
		       WHERE CALL_VERSION = '%s'", bnm);
	res = PQexec(conn, sql);
	checkselectstatus(res);

	for (ntup = 0; ntup < PQntuples(res); ntup++)
	{
		fprintf(fp, "INVOKE\t%d\t%d\n", atol(PQgetvalue(res, ntup, 0)), 
				atol(PQgetvalue(res, ntup, 1)));
	}
	PQclear(res);
}

void listbnm()
{
	PGresult *res;
	char sql[512];
	int ntup;

	begintransaction();
	
	{
		sprintf(sql, "SELECT DISTINCT PS_VERSION FROM BTV_MAKEPROC");
		res = PQexec(conn, sql);
		checkselectstatus(res);

		printf("\n");
		for (ntup = 0; ntup < PQntuples(res); ntup ++)
		{
			printf("%s\n", PQgetvalue(res, ntup, 0));
		}
		printf("\n");
		PQclear(res);
	}
	
	endtransaction();
}
