
#include <assert.h>
#include <string.h>
#include "util.h"

static long long memory = 0;

void *xmalloc(unsigned int size)
{
	void *result = (void *)malloc(size);
	memory += size;
	if (result == 0)
	{
		printf("virtual memory exhausted, total %d momeory are used.\n", memory);
		assert(0);
	}
	return result;
}

char *xstrdup(const char *ptr)
{
	char *result = (char *) xmalloc (strlen(ptr) + 1);
	strcpy(result, ptr);
	result[strlen(ptr)] = '\0';
	return result;
}

char *xstrncpy(const char *ptr, int len)
{
	char *result = (char *) xmalloc (len + 1);
	strncpy(result, ptr, len);
	result[len] = '\0';
	return result;
}

char *xstr3cat(const char *ptr1, const char c, const char *ptr3)
{
	int size = strlen(ptr1) + 1 + strlen(ptr3);
	char *result = (char *) xmalloc (size + 1) ;

	sprintf(result, "%s%c%s", ptr1, c, ptr3);
	result[size] = '\0';
	return result;
}

char *xtrim(const char * str)
{
        const char *c;
        char *result;
        char *r;

        c = str;
	while (*c == ' ' && c < str + strlen(str)) c++;

	if (c == str + strlen(str)) return xstrdup("");

	result = xstrdup(c);	
	r = result + strlen(result) - 1;
			
	while (*r == ' ' || *r == '\x0') r--;
	*(r + 1) = '\x0';
	return result;
}


void print_memory()
{
	printf("xmalloced memory = %d\n", memory);
}

char *get_input_string(const char *msg)
{
        char *res = (char *)xmalloc(256);
        printf("\n%s =>", msg);
        scanf("%s", res);
        return res;
}

struct StackBucket * StackBucket_(void * c, struct StackBucket *ls)
{
	struct StackBucket * res = (struct StackBucket*)xmalloc(sizeof(*res));
	res->content = c;
	res->next = ls;
	return res;
}

void stackbucket_free(struct StackBucket *b)
{
	free (b->content);
	free(b);
}

struct Stack * Stack_()
{
	struct Stack *res = (struct Stack *)xmalloc(sizeof(*res));
	res->number = 0;
	res->top = 0;
	return res;
}

void stack_push(struct Stack * s, void * c)
{
	s->number ++;
	s->top = StackBucket_(c, s->top);
}

void * stack_pop(struct Stack * s)
{
	struct StackBucket *b;
	void * res;
	
	if (s->number <= 0) return 0;
	
	b = s->top;
	s->number --;
	s->top = b->next;
	res = b->content;
	free (b);
	return res;
}

int stack_isempty(struct Stack * s)
{
	return s->number;
}

void stack_free(struct Stack *s)
{
	struct StackBucket *b;

	while (1) {
		b = stack_pop(s);
		if (b != 0)
			stackbucket_free(b);
		else
			break;
	}
}

