
#include <stdio.h>
#include <dirent.h>

int cmpfile (const char *fn1, const char *fn2)
{
	FILE * f1 = fopen(fn1, "r");
	FILE * f2 = fopen(fn2, "r");

	char buf1[512], buf2[512];
	int len1, len2;

	if (!f1 || !f2)
	{
		printf("Can't open file %s or %s", fn1, fn2);
		abort();
	}
	
	len1=fread(buf1, 1, 512, f1);
	len2=fread(buf2, 1, 512, f2);

	while (len1 == len2 && len1 > 0)
	{
		if (strncmp(buf1, buf2, len1)) return 1;
		
		len1=fread(buf1, 1, 512, f1);
	       	len2=fread(buf2, 1, 512, f2);
	}

	fclose(f1);
	fclose(f2);

	if (len1 == len2) return 0;
	else return 1;
}

int find_file (const char *f)
{
	FILE *fp = fopen (f, "r");
	if (fp == 0) return 1;

	fclose (fp);
	return 0;
}

int find_dir (const char *d)
{
	DIR *dp = opendir (d);
	if (d == NULL) return 1;

	closedir (dp);
	return 0;
}

int cmpdir(const char *dn1, const char *dn2)
{
	struct dirent *direntp;
	DIR *dirp;
	char *buf1, *buf2;
	
	dirp = opendir (dn1);
	while ((direntp = readdir (dirp)) != NULL )
	{
		if (strncmp(direntp->d_name, ".", 1) == 0) continue;

		buf1 = (char *)malloc(sizeof(dn1) + sizeof(direntp->d_name)+1);
		buf2 = (char *)malloc(sizeof(dn2) + sizeof(direntp->d_name)+1);
		sprintf(buf1, "%s/%s", dn1, direntp->d_name);
		sprintf(buf2, "%s/%s", dn2, direntp->d_name);
		
		switch (direntp->d_type)
		{
			case 4:
				if (find_dir (buf2) == 0)
				{
					cmpdir(buf1, buf2);
				}
				break;
			case 8:
				if (find_file (buf2) == 0)
				{
					if (cmpfile(buf1, buf2) == 0)
						printf("match %s, %s\n", buf1, buf2);
					else 
						printf("notmatch %s, %s\n", buf1, buf2);
				}
				break;
			default:
				printf("file = %s, type = %d\n", buf1, direntp->d_type);
				break;
		}
		free (buf1);
		free (buf2);
	}
	closedir( dirp );
}

int main(int argc, char **argv)
{
	if (argc < 3)
	{
		printf("\nUsage: cmpdir dir1 dir2\n\n");
	}
	else
	{
		cmpdir(argv[1], argv[2]);
	}
	return (0);
}

