
#include "query.h"
#include <assert.h>

PGconn *conn;
FILE *outfp;

#define exec_select(res, sql)\
	res = PQexec(conn, sql);\
	if (res == NULL)\
                die("meet error when SELECT command is being executed");\
        else if (PQresultStatus(res) != PGRES_TUPLES_OK)\
        { \
		PQclear(res); \
		die(sql); \
	}\
	if (PQntuples(res) < 1)\
	{\
		PQclear(res);\
		die("Target doesn't exist");\
	}

static compare_target(const char *oid_a, const char *oid_b)
{
	char sql[512];
	PGresult *res_a, *res_b;
	char *name_a, *name_b, *dir_a, *dir_b;
	char *cmdstxt_a, *cmdstxt_b, *execmds_a, *execmds_b;
	int isphony_a,isphony_b, istarget_a,istarget_b, isdblcln_a,isdblcln_b;
	
	char *c_sql = "SELECT INST_NAME, INST_CMDSTXT, INST_EXECMDS,\
			      INST_ISPHONY, INST_ISTARGET, INST_ISDBLCLN,\
			      INST_DIR \
			 FROM BTV_FILEINST\
			WHERE OID = %s";

	sprintf(sql, c_sql, oid_a);
	exec_select(res_a, sql);
	sprintf(sql, c_sql, oid_b);
	exec_select(res_b, sql);

	name_a    = PQgetvalue(res_a, 0, 0);
	dir_a     = PQgetvalue(res_a, 0, 6);
	if (PQgetisnull(res_a, 0, 1)) 
		cmdstxt_a = 0;
	else
		cmdstxt_a = PQgetvalue(res_a, 0, 1);
	if (PQgetisnull(res_a, 0, 2))
		execmds_a = 0;
	else
		execmds_a = PQgetvalue(res_a, 0, 2);
	isphony_a = atoi(PQgetvalue(res_a, 0, 3));
	if (!isphony_a) isphony_a = 1;
	istarget_a= atoi(PQgetvalue(res_a, 0, 4));
	if (!istarget_a) istarget_a = 1;
	isdblcln_a= atoi(PQgetvalue(res_a, 0, 5));
	if (!isdblcln_a) isdblcln_a = 1;

	name_b    = PQgetvalue(res_b, 0, 0);
	dir_b     = PQgetvalue(res_b, 0, 6);
	if (PQgetisnull(res_b, 0, 1))
		cmdstxt_b = 0;
	else 
		cmdstxt_b = PQgetvalue(res_b, 0, 1);
	if (PQgetisnull(res_b, 0, 2))
		execmds_b = 0;
	else
		execmds_b = PQgetvalue(res_b, 0, 2);
	isphony_b = atoi(PQgetvalue(res_b, 0, 3));
	if (!isphony_b) isphony_b = 1;
	istarget_b= atoi(PQgetvalue(res_b, 0, 4));
	if (!istarget_b) istarget_b = 1;
	isdblcln_b= atoi(PQgetvalue(res_b, 0, 5));
	if (!isdblcln_b) isdblcln_b = 1;

	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, "Comparing Target\n\t%s/%s\n\t%s/%s\n", 
			dir_a, name_a, dir_b, name_b);

	if ((!cmdstxt_a && !cmdstxt_b) || 
		(cmdstxt_a && cmdstxt_b && strcmp(cmdstxt_a, cmdstxt_b) == 0))
		fprintf(outfp, " \nCommand text: SAME\n");
	else 
	{
		fprintf (outfp, " \nCommand text for %s/%s:\n", dir_a, name_a);
		print_cmd(cmdstxt_a);
		fprintf (outfp, " \nCommand text for %s/%s:\n", dir_b, name_b);
		print_cmd(cmdstxt_b);
	}

	if ((!execmds_a && !execmds_b) || 
		(execmds_a && execmds_b && strcmp(execmds_a, execmds_b) == 0))
		fprintf(outfp, " \nExecuted commands: SAME\n");
	else 
	{
		fprintf (outfp, " \nExecuted commands for %s/%s:\n", 
				dir_a, name_a);
		print_cmd(execmds_a);
		fprintf (outfp, " \nExecuted commands for %s/%s:\n", 
				dir_b, name_b);
		print_cmd(execmds_b);
	}

	if (isphony_a != isphony_b)
	{
		if (isphony_a)
			fprintf(outfp, "%s/%s is PHONY target\n",dir_a,name_a);
		if (isphony_b)
			fprintf(outfp, "%s/%s is PHONY target\n",dir_b,name_b);
	}

	if (istarget_a != istarget_b)
	{
		if (istarget_a)
			fprintf(outfp, "%s/%s is a target in Makefile\n",
					dir_a, name_a);
		if (istarget_b)
			fprintf(outfp, "%s/%s is target in Makefile\n", 
					dir_b, name_b);
	}

	if (isdblcln_a != isdblcln_b)
	{
		if (isdblcln_a)
			fprintf(outfp, "%s/%s has double colon rule\n", 
					dir_a, name_a);
		if (isdblcln_b)
			fprintf(outfp, "%s/%s has double colon rule\n", 
					dir_b, name_b);
	}

	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " \n");

}

int main(int argc, char **argv, char**envp)
{
	int atup, btup, ntup, cmp_result;

	if (argc < 4)
	{
		printf("Environment Comparison: Do Not have ENOUGH params. \n");
		printf("Do nothing!\n");
		return 0;
	}

	outfp = fopen(argv[1], "w");
	if (!outfp) {
		printf("Can not open file.\n");
		return 1;
	}
	
	initdbconnection();
	begintransaction();
	
	compare_target(argv[2], argv[3]);

	endtransaction();
	enddbconnection();
	fprintf(outfp, "\n");
	close (outfp);
}
#undef exec_select
