
#include "query.h"
#include <assert.h>

PGconn *conn;
FILE *outfp;

#define exec_select(res, sql)\
	res = PQexec(conn, sql);\
	if (res == NULL)\
                die("meet error when SELECT command is being executed");\
        else if (PQresultStatus(res) != PGRES_TUPLES_OK)\
        { \
		PQclear(res); \
		die(sql); \
	}\
	if (PQntuples(res) < 1)\
	{\
		die("There no required environment definition");\
	}

static sort(int *arr, int len, PGresult *res)
{
	int i, j, temp;
	for (i = 0; i < len; i++)
	{
		arr[i] = i;
		j = i;
		while (j > 0)
		{
			if (xstrcmp(	PQgetvalue(res, arr[j], 1), 
					PQgetvalue(res, arr[j-1], 1) ) < 0)
			{
				temp = arr[j];
				arr[j] = arr[j-1];
				arr[j-1] = temp;
			}
			j --;
		}
	}
	
}

static compare_var(const char *oid_a, const char *oid_b)
{
	char sql[512];
	PGresult *res_a, *res_b;
	int ai, bi, ntup_a, ntup_b, cmp_result;
	int *arr_a, *arr_b;

	char *c_sql ="SELECT BTV_FILEINST.INST_NAME, \
			     VAR_NAME, VAR_VALUE, VAR_SOURCE\
			FROM BTV_FILEINST, BTV_VARIABLE\
		       WHERE BTV_FILEINST.OID = %s\
		         AND INST_PROC = VAR_PROC\
			 AND VAR_SOURCE IN (1, 2, 3, 4, 5, 6)\
		       ORDER BY VAR_NAME, VAR_SOURCE";
	
	sprintf(sql, c_sql, oid_a);
	exec_select(res_a, sql);
	sprintf(sql, c_sql, oid_b);
	exec_select(res_b, sql);

	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " Comparing Environment at %s and that at %s\n", 
			PQgetvalue(res_a, 0, 0), PQgetvalue(res_b, 0 ,0));
	
	ntup_a = PQntuples(res_a);
	ntup_b = PQntuples(res_b);
	arr_a = (int *)xmalloc(sizeof(int) * ntup_a);
	arr_b = (int *)xmalloc(sizeof(int) * ntup_b);
	sort(arr_a, ntup_a, res_a);
	sort(arr_b, ntup_b, res_b);
	
	ai = 0; bi = 0;
	while (ai < ntup_a && bi < ntup_b)
	{
		cmp_result = xstrcmp(	PQgetvalue(res_a, arr_a[ai], 1), 
					PQgetvalue(res_b, arr_b[bi], 1));
		
		if (cmp_result < 0)	/* a < b */
		{
			print_var(atoi (PQgetvalue(res_a, arr_a[ai], 3)),
					PQgetvalue(res_a, arr_a[ai], 1),
					PQgetvalue(res_a, arr_a[ai], 2), "1>" );
			ai ++;
		}
		else if (cmp_result == 0)	/* equal */
		{
			if (xstrcmp( PQgetvalue(res_a, arr_a[ai], 2), 
				     PQgetvalue(res_b, arr_b[bi], 2)) == 0)
			{
				ai ++; bi ++;
				continue;
			}
			else
			{
				print_var(atoi (PQgetvalue(res_a,arr_a[ai],3)),
						PQgetvalue(res_a,arr_a[ai],1),
						PQgetvalue(res_a,arr_a[ai],2), "1>");
				print_var(atoi (PQgetvalue(res_b,arr_b[bi],3)),
						PQgetvalue(res_b,arr_b[bi],1),
						PQgetvalue(res_b,arr_b[bi],2), "2<");
				ai ++; bi ++;
			}
		}
		else	/* a > b */
		{
			print_var(atoi (PQgetvalue(res_b, arr_b[bi], 3)),
					PQgetvalue(res_b, arr_b[bi], 1),
					PQgetvalue(res_b, arr_b[bi], 2), "2<");
			bi ++;
		}
	}
	while (ai < ntup_a)
	{
		print_var(atoi (PQgetvalue(res_a, arr_a[ai], 3)),
				PQgetvalue(res_a, arr_a[ai], 1),
				PQgetvalue(res_a, arr_a[ai], 2), "1>" );
		ai ++;
	}
	while (bi < ntup_b)
	{
		print_var(atoi (PQgetvalue(res_b, arr_b[bi], 3)),
				PQgetvalue(res_b, arr_b[bi], 1),
				PQgetvalue(res_b, arr_b[bi], 2), "2<");
		bi ++;
	}
	
	PQclear(res_a);
	PQclear(res_b);
	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " \n");
}

int main(int argc, char **argv, char**envp)
{
	if (argc < 4)
	{
		printf("Environment Comparison: Do Not have ENOUGH params. \n");
		printf("Do nothing!\n");
		return 0;
	}

	outfp = fopen(argv[1], "w");
	if (!outfp) {
		printf("Can not open file");
		return 1;
	}
	
	initdbconnection();
	begintransaction();
	
	if (argc == 4)
		compare_var(argv[2], argv[3]);
	else if (argc == 6)
	{
		compare_var(argv[2], argv[4]);
		compare_var(argv[3], argv[5]);
	}

	endtransaction();
	enddbconnection();
	fprintf(outfp, "\n");
	close (outfp);
}
 
#undef exec_select
