
#include "query.h"

PGconn *conn;
FILE *outfp;

static query_cmd(const char *oid)
{
	PGresult *res;
	char sql[512];
	
	sprintf(sql, "SELECT INST_NAME, INST_DIR, INST_CMDSTXT, INST_EXECMDS\
			FROM BTV_FILEINST\
		       WHERE OID = %s", oid);
	res = PQexec(conn, sql);
        if (res == NULL)
                die("meet error when SELECT command is being executed"); 
        else if (PQresultStatus(res) != PGRES_TUPLES_OK)
        { 
		PQclear(res); 
		die(sql); 
	}

	if (PQntuples(res) < 1)
	{
		die("There no such file, the data file may not be up-to-date");
	}

	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " \nCommand information for %s/%s\n", 
			PQgetvalue(res, 0, 1), PQgetvalue(res, 0, 0));
	
	fprintf(outfp, " \n------------------------------------------\n");
	fprintf(outfp, " \nCOMMAND TEXT:\n");
	if (PQgetisnull(res, 0, 2))
		fprintf(outfp, "\tNo defined command for this node\n");
	else
		print_cmd(PQgetvalue(res, 0, 2));

	fprintf(outfp, " \n------------------------------------------\n");
	fprintf(outfp, " \nEXECUTED COMMAND:\n");
	if (PQgetisnull(res, 0, 3))
		fprintf(outfp,"\tNo command is executed to update this node\n");
	else
		print_cmd(PQgetvalue(res, 0, 3));
	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " \n");

	PQclear(res);
}

int main(int argc, char **argv, char**envp)
{
	int i;

	if (argc < 3)
	{
		fprintf(outfp, "Command Query: Not enough params.\n");
	       	fprintf(outfp, "               Do nothing!\n");
		return 0;
	}

	outfp = fopen(argv[1], "w");
	if (!outfp) {
		fprintf(outfp, "Can not open file");
		return 1;
	}
	
	initdbconnection();
	begintransaction();

	for (i = 2; i < argc; i ++)
		query_cmd(argv[i]);

	endtransaction();
	enddbconnection();
	fprintf(outfp, "\n");
	close (outfp);
}
