
#include "query.h"
#include <assert.h>

void *xmalloc(unsigned int size)
{
        void *result = (void *)malloc(size);
        if (result == 0) assert(0);
        return result;
}


static char *xstrdup(const char *ptr)
{
        char *result = (char *) xmalloc (strlen(ptr) + 1);
        strcpy(result, ptr);
        result[strlen(ptr)] = '\0';
        return result;
}

static char *xstrncpy(const char *ptr, int len)
{
        char *result = (char *) xmalloc (len + 1);
        strncpy(result, ptr, len);
        result[len] = '\0';
        return result;
}

void die(const char *msg)
{
	fprintf(outfp, "%s\n", msg);
	fprintf(outfp, "\t%s\n", PQerrorMessage(conn));
	fprintf(outfp, "\n");
	PQfinish(conn);		
	exit(1);
}

void initdbconnection()
{
        /* backend connection */
        char *pghost = NULL, *pgport = NULL, *pgoptions = NULL, *pgtty = NULL;
        /*char *dbname = "POSTGRESQL";*/
        /*char *dbname = "BTVDB";*/
        /*char *dbname = "GCC";*/
        /*char *dbname = "MOZILLA";*/
        /*char *dbname = "LINUX";*/
        char *dbname = "BTVDB";

        /* make a connection to the database */
        conn = PQsetdb(pghost, pgport, pgoptions, pgtty, dbname);

        /* check to see that the backend connection was successfully made */
        if (PQstatus(conn) == CONNECTION_BAD) 
		die("Failed to build a connection");
}

void enddbconnection()
{
        PQfinish(conn);
}

void begintransaction() 
{
       /* start transaction before export */
       PGresult *res;
       res = PQexec(conn, "BEGIN");
       if (PQresultStatus(res) != PGRES_COMMAND_OK)
       {
	      PQclear(res);
	      die("Failed to begin transaction");
       }
       PQclear(res);
}

void endtransaction()
{ 
	/* end transaction */
	PGresult *res;
	res = PQexec(conn, "END");
	PQclear(res);
}

static char *xtrim(const char * str)
{
	const char *c;
	char *result;
	char *r;

	c = str;
	while (*c == ' ' && c < str + strlen(str)) c++;
	
	if (c == str + strlen(str)) return xstrdup("");
	
	result = xstrdup(c);
	r = result + strlen(result) - 1;

	while (*r == ' ' || *r == '\x0') r--;
	*(r + 1) = '\x0';
	return result;
}

int xstrcmp(const char *str1, const char *str2)
{
	char *xstr1, *xstr2;
	int result;
	
	if (!str1 && !str2) return 0;
	
	if (str1 && str2)
	{
		xstr1 = xtrim(str1); 
		xstr2 = xtrim(str2);
	}
	else
		return 1;

	result = strcmp(xstr1, xstr2);

	free (xstr1);
	free (xstr2);
	return result;
}

void print_var(const int source, const char *name, const char *value, char *prefix)
{
        char *display_name;
        char *strend = (char *)strchr(name, ' ');
        const int linelen = 60;
        char varline[linelen + 1];
        const char *v;
        int i;

	if (strend < name + strlen(name) - 3)
        {
                if (*(strend + 1) == ' ')
                        display_name = xstrncpy(name, strend - name);
                else
                        display_name = xstrncpy(name, strend - name + 2);
        }
        else
        {
                display_name = xstrdup(name);
        }

        sprintf(varline, "%s %d %s=", prefix, source, display_name);
        free (display_name);

	i = strlen(varline);
        v = value;

	while (v < value + strlen(value))
	{
		if (i < linelen)
		{
			varline[i] = *v;
			i++;
			v++;
		}			
		else {
			varline[i] = '\0';
			fprintf(outfp, "%s\n", varline);
			i = 1;
			varline[0] = '\t';			
			v ++;
		}
	}	
	if (i < linelen)
	{		
		varline[i] = '\0';	
		fprintf(outfp, "%s\n", varline);				
	}
}

void print_cmd(const char * cmd)
{
        const char * c = cmd;
        const int linelen = 60;
        char cmdline[linelen + 1];
        int i = 0;

	if (cmd == 0)
	{
		fprintf(outfp, " NONE.\n");
		return;
	}
	
	while (c < cmd + strlen(cmd))
      	{
		if (*c != '\n' && i < linelen)
		{
			cmdline[i] = *c;
			i++;
			c++;
		}
		else {
			cmdline[i] = '\0';
			fprintf(outfp, "\t%s\n", cmdline);
			cmdline[0] = '\0';
			i = 0;
			if (*c == '\n') c++;
		}
	}
	if (strlen(cmdline) > 0 && i <= linelen)
	{
		cmdline[i] = '\0';
		fprintf(outfp, "\t%s\n", cmdline);
	}
}



