
#include "query.h"
#include <assert.h>

PGconn *conn;
FILE *outfp;

static query_var(const char *oid)
{
	char sql[512];
	PGresult *res;
	int ntup;

	sprintf(sql, "SELECT VAR_NAME, VAR_VALUE, VAR_SOURCE\
			FROM BTV_FILEINST, BTV_VARIABLE\
		       WHERE BTV_FILEINST.OID = %s\
		         AND INST_PROC = VAR_PROC\
			 AND VAR_SOURCE IN (1, 2, 3, 4, 5, 6)\
		       ORDER BY VAR_SOURCE, VAR_NAME", oid);
	res = PQexec(conn, sql);
        if (res == NULL)
                die("meet error when SELECT command is being executed"); 
        else if (PQresultStatus(res) != PGRES_TUPLES_OK)
        { 
		PQclear(res); 
		die(sql); 
	}

	if (PQntuples(res) < 1)
	{
		die("There no required environment definition");
	}

	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " Environment at %s from %s\n", oid, "all");
	for (ntup = 0; ntup < PQntuples(res); ntup++)
	{
		print_var(atoi (PQgetvalue(res, ntup, 2)), 
				PQgetvalue(res, ntup, 0),
				PQgetvalue(res, ntup, 1), " ");
	}
	
	fprintf(outfp, " \n==========================================\n");
	fprintf(outfp, " \n");
	PQclear(res);
}

int main(int argc, char **argv, char**envp)
{
	int i;

	if (argc < 3)
	{
		printf("Environment Query: Do Not have ENOUGH params. \n");
		printf("Do nothing!\n");
		return 0;
	}

	outfp = fopen(argv[1], "w");
	if (!outfp) {
		printf("Can not open file");
		return 1;
	}
	
	initdbconnection();
	begintransaction();

	for (i = 2; i < argc; i++)
		query_var(argv[i]);

	endtransaction();
	enddbconnection();
	fprintf(outfp, "\n");
	close (outfp);
}
